# cosmic.rb -- Cosmic Ruby Library
# various convenience methods and helpful extension to the standard Ruby
#   library.
#
# written 010729 by Mirian Crzig Lennox
# Copyright (c) 2001, released under the terms of the GNU Public License

class File
  def File.backup fn
    version = 0
    new_fn = fn
    while (File.exists? new_fn)
      version += 1
      new_fn = fn + "." + (version.to_s)
    end
    File.rename fn, new_fn unless version == 0
  end

  def File.create fn
    File.backup fn
    File.open fn, "w"
  end
end

class Integer
  def odd?
    self % 2 == 1
  end

  def even?
    self % 2 == 0
  end

  def howmany d
    (self / d) + (((self % d) > 0) ? 1 : 0)
  end

  def roundup n
    (self%n == 0) ? self : (self + (n - self%n))
  end
end

class String
  def unpack1 fmt
    unpack(fmt).shift
  end
end

def getargs(*names)
  if ARGV.length != names.length
    $stderr.print "usage: #{$0}"
    names.each do |name|
      $stderr.print " <", name, ">"
    end
    $stderr.print "\n"
    exit -1
  end
  ARGV
end

def mask(num)
  (1 << num) - 1
end

def splice num_array, *bit_array
  return 0 unless num_array
  word = 0
  i = 0
  bit_array.each do |shft|
    num = num_array[i]
    unless num
      num = 0
    end
    i += 1
    word = (word << shft) | (num & mask(shft))
  end
  return word
end

def shred word, *bit_array
  arr = []
  bit_array.reverse.each do |shft|
    arr.unshift(word & mask(shft))
    word >>= shft
  end
  return arr
end

