# dumper.rb -- DUMPER archive tape format
#
# written 010810 by Mirian Crzig Lennox
# Copyright (c) 2001, released under the terms of the GNU Public License

require "cosmic"
require "pdp10"
require "tape"

module PDP10
  module Dumper
    include PDP10
    include Tape

    def create_tape_header tapenum
      @dumper_tapenum = tapenum << 18
      @dumper_tapedate = to_sixbit(Time.new.strftime("%y%m%d"))
      @dumper_wroteheader = false
    end

    def restore_file
      filebuf = read_file
      return nil unless filebuf
      hdr = filebuf.shift
      tapenum = nil
      tapedate = nil
      if (hdr == 0777774000000)
	tapenum = (filebuf.shift) >> 18
	tapedate = from_sixbit(filebuf.shift)
	filebuf.shift
	hdr = filebuf.shift
      end
      if (hdr == 0777775000000)
	filebuf.shift
	dirname = from_sixbit(filebuf.shift)
	fn1 = nil
	fn2 = nil
	filedate = nil
      elsif (hdr == 0777772000000)
	dirname = from_sixbit(filebuf.shift)
	fn1 = from_sixbit(filebuf.shift)
	fn2 = from_sixbit(filebuf.shift)
	filebuf.shift
	filedate = from_sixbit(filebuf.shift)
      else
	$stdout.printf "invalid file header %012o\n", hdr
	fail
      end
      return [ filebuf, dirname, fn1, fn2, filedate, tapenum, tapedate ]
    end

    def dump_file buf, dirname, fn1, fn2, create_time
      filebuf = []
      unless @dumper_wroteheader
	filebuf.push(0777774000000, @dumper_tapenum, @dumper_tapedate,0)
	@dumper_wroteheader = true
      end
      filebuf.push(0777772000000,to_sixbit(dirname),to_sixbit(fn1),
                   to_sixbit(fn2), 0,
		   to_sixbit(create_time.strftime("%y%m%d")))
      unless (buf.empty?)
	filebuf.push(*buf)
      end
      write_file(filebuf)
    end

    def dump_link dirname, fn1, fn2, create_time, linkdirname, linkfn1, linkfn2
      filebuf = []
      unless @dumper_wroteheader
	filebuf.push(0777774000000, @dumper_tapenum, @dumper_tapedate,0)
	@dumper_wroteheader = true
      end
      filebuf.push(0777772000000,
                   to_sixbit(dirname),to_sixbit(fn1), to_sixbit(fn2),
		   01000000,
		   to_sixbit(create_time.strftime("%y%m%d")),
                   to_sixbit(linkdirname),to_sixbit(linkfn1),
                   to_sixbit(linkfn2))
      write_file(filebuf)
    end

  end
end
